import pytest
import datetime
from unittest.mock import Mock, patch
from thousandeyes_sdk.core.api_client import ApiClient
from thousandeyes_sdk.core.configuration import Configuration
from thousandeyes_sdk.core import rest
from thousandeyes_sdk.core.exceptions import ApiException


@pytest.fixture
def api_client():
    config = Configuration()
    return ApiClient(configuration=config)


def test_api_client_initialization(api_client):
    assert api_client.configuration is not None
    assert isinstance(api_client.rest_client, rest.RESTClientObject)
    assert api_client.default_headers == {}


def test_set_default_header(api_client):
    api_client.set_default_header('X-Test-Header', 'test_value')
    assert api_client.default_headers['X-Test-Header'] == 'test_value'


def test_user_agent_property(api_client):
    api_client.user_agent = 'test-agent'
    assert api_client.user_agent == 'test-agent'


def test_get_default():
    default_client = ApiClient.get_default()
    assert isinstance(default_client, ApiClient)


def test_set_default():
    new_default = ApiClient()
    ApiClient.set_default(new_default)
    assert ApiClient.get_default() == new_default


def test_param_serialize(api_client):
    method, url, headers, _, __ = api_client.param_serialize(
        method='GET',
        resource_path='/test/{id}',
        path_params={'id': 1},
        query_params={'aid': '12'},
        header_params={'X-Test': 'test_value'}
    )
    assert method == 'GET'
    print(url + "3")
    assert url == api_client.configuration.host + '/test/1?aid=12'
    assert headers['X-Test'] == 'test_value'


@patch('thousandeyes_sdk.core.rest.RESTClientObject.request')
def test_call_api(mock_request, api_client):
    mock_response = Mock()
    mock_response.data = b'{"dummyKey": "someValue"}'
    mock_response.status = 200
    mock_request.return_value = mock_response

    response = api_client.call_api(
        method='GET',
        url='/tests',
        body=None,
        post_params=None,
        _request_timeout=None
    )
    assert response.data == b'{"dummyKey": "someValue"}'
    assert response.status == 200


@patch('thousandeyes_sdk.core.rest.RESTClientObject.request')
def test_call_api_exception(mock_request, api_client):
    mock_request.side_effect = ApiException(status=404, reason="Not Found")
    with pytest.raises(ApiException):
        api_client.call_api(
            method='GET',
            url='/tests',
            body=None,
            post_params=None,
            _request_timeout=None
        )


def test_response_deserialize(api_client):
    mock_response = Mock()
    mock_response.data = b'{"dummyKey": "someValue"}'
    mock_response.status = 200
    mock_response.getheader.return_value = 'application/json'
    mock_response.getheaders.return_value = {}

    response = api_client.response_deserialize(
        response_data=mock_response,
        response_types_map={'200': 'dict'},
        models={}
    )
    assert response.data == {"dummyKey": "someValue"}
    assert response.status_code == 200


def test_sanitize_for_serialization(api_client):
    data = {
        'str': 'value',
        'int': 1,
        'float': 1.1,
        'bool': True,
        'datetime': datetime.datetime(2023, 1, 1),
        'date': datetime.date(2023, 1, 1),
        'list': [1, 2, 3],
        'dict': {'key': 'value'}
    }
    sanitized = api_client.sanitize_for_serialization(data)
    assert sanitized['str'] == 'value'
    assert sanitized['int'] == 1
    assert sanitized['float'] == 1.1
    assert sanitized['bool'] is True
    assert sanitized['datetime'] == '2023-01-01T00:00:00'
    assert sanitized['date'] == '2023-01-01'
    assert sanitized['list'] == [1, 2, 3]
    assert sanitized['dict'] == {'key': 'value'}



import pytest
from pydantic import ValidationError
from thousandeyes_sdk.core.api_response import ApiResponse


def test_api_response_valid():
    response = ApiResponse(
        status_code=200,
        headers={"Content-Type": "application/json"},
        data={"key": "value"},
        raw_data=b'{"key": "value"}'
    )
    assert response.status_code == 200
    assert response.headers == {"Content-Type": "application/json"}
    assert response.data == {"key": "value"}
    assert response.raw_data == b'{"key": "value"}'


def test_api_response_missing_optional_headers():
    response = ApiResponse(
        status_code=200,
        data={"key": "value"},
        raw_data=b'{"key": "value"}'
    )
    assert response.status_code == 200
    assert response.headers is None
    assert response.data == {"key": "value"}
    assert response.raw_data == b'{"key": "value"}'


def test_api_response_invalid_status_code():
    with pytest.raises(ValidationError):
        ApiResponse(
            status_code="200",  # Invalid type
            headers={"Content-Type": "application/json"},
            data={"key": "value"},
            raw_data=b'{"key": "value"}'
        )


def test_api_response_invalid_raw_data():
    with pytest.raises(ValidationError):
        ApiResponse(
            status_code=200,
            headers={"Content-Type": "application/json"},
            data={"key": "value"},
            raw_data="raw data"  # Invalid type
        )



# Copyright 2024 Cisco Systems, Inc. and its affiliates
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

import copy
import http.client as httplib
import logging
import multiprocessing
import sys
from logging import FileHandler
from typing import Optional

import urllib3

from thousandeyes_sdk.core.thousandeyes_retry import ThousandEyesRetry


class Configuration:

    _default = None

    def __init__(self, host=None,
                 access_token=None,
                 ssl_ca_cert=None,
                 retries=None
                 ) -> None:
        """Constructor
        """
        self._base_path = "https://api.thousandeyes.com/v7" if host is None else host
        """Default Base url
        """
        self.access_token = access_token
        """Access token
        """
        self.logger = {}
        """Logging Settings
        """
        self.logger["package_logger"] = logging.getLogger("admin")
        self.logger["urllib3_logger"] = logging.getLogger("urllib3")
        self.logger_format = '%(asctime)s %(levelname)s %(message)s'
        """Log format
        """
        self.logger_stream_handler = None
        """Log stream handler
        """
        self.logger_file_handler: Optional[FileHandler] = None
        """Log file handler
        """
        self.logger_file = None
        """Debug file location
        """
        self.verify_ssl = True
        """SSL/TLS verification
           Set this to false to skip verifying SSL certificate when calling API
           from https server.
        """
        self.ssl_ca_cert = ssl_ca_cert
        """Set this to customize the certificate file to verify the peer.
        """
        self.cert_file = None
        """client certificate file
        """
        self.key_file = None
        """client key file
        """
        self.assert_hostname = None
        """Set this to True/False to enable/disable SSL hostname verification.
        """
        self.tls_server_name = None
        """SSL/TLS Server Name Indication (SNI)
           Set this to the SNI value expected by the server.
        """
        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5
        """urllib3 connection pool's maximum number of connections saved
           per pool. urllib3 uses 1 connection as default value, but this is
           not the best value when you are making a lot of possibly parallel
           requests to the same host, which is often the case here.
           cpu_count * 5 is used as default value to increase performance.
        """
        self.proxy: Optional[str] = None
        """Proxy URL
        """
        self.proxy_headers = None
        """Proxy headers
        """
        self.retries = ThousandEyesRetry()
        if retries:
            self.retries = retries
        """Adding retries to override urllib3 default value 3
        """
        self.socket_options = None
        """Options to pass down to the underlying urllib3 socket
        """
        self.datetime_format = "%Y-%m-%dT%H:%M:%S.%f%z"
        """datetime format
        """
        self.date_format = "%Y-%m-%d"
        """date format
        """

    def __deepcopy__(self, memo):
        cls = self.__class__
        result = cls.__new__(cls)
        memo[id(self)] = result
        for k, v in self.__dict__.items():
            if k not in ('logger', 'logger_file_handler'):
                setattr(result, k, copy.deepcopy(v, memo))
        # shallow copy of loggers
        result.logger = copy.copy(self.logger)
        # use setters to configure loggers
        result.logger_file = self.logger_file
        return result

    def __setattr__(self, name, value):
        object.__setattr__(self, name, value)

    @classmethod
    def set_default(cls, default):
        """Set default instance of configuration.

        It stores default configuration, which can be
        returned by get_default_copy method.

        :param default: object of Configuration
        """
        cls._default = default

    @classmethod
    def get_default(cls):
        """Return the default configuration.

        This method returns newly created, based on default constructor,
        object of Configuration class or returns a copy of default
        configuration.

        :return: The configuration object.
        """
        if cls._default is None:
            cls._default = Configuration()
        return cls._default

    @property
    def logger_file(self):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        return self.__logger_file

    @logger_file.setter
    def logger_file(self, value):
        """The logger file.

        If the logger_file is None, then add stream handler and remove file
        handler. Otherwise, add file handler and remove stream handler.

        :param value: The logger_file path.
        :type: str
        """
        self.__logger_file = value
        if self.__logger_file:
            # If set logging file,
            # then add file handler and remove stream handler.
            self.logger_file_handler = logging.FileHandler(self.__logger_file)
            self.logger_file_handler.setFormatter(self.logger_formatter)
            for _, logger in self.logger.items():
                logger.addHandler(self.logger_file_handler)

    @property
    def logger_format(self):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        return self.__logger_format

    @logger_format.setter
    def logger_format(self, value):
        """The logger format.

        The logger_formatter will be updated when sets logger_format.

        :param value: The format string.
        :type: str
        """
        self.__logger_format = value
        self.logger_formatter = logging.Formatter(self.__logger_format)

    def auth_settings(self):
        """Gets Auth Settings dict for api client.

        :return: The Auth Settings information dict.
        """
        auth = {}
        if self.access_token is not None:
            auth['BearerAuth'] = {
                'type': 'bearer',
                'in': 'header',
                'key': 'Authorization',
                'value': 'Bearer ' + self.access_token
            }
        return auth

    @property
    def host(self):
        """Return generated host."""
        return self._base_path

    @host.setter
    def host(self, value):
        """Fix base path."""
        self._base_path = value



# Copyright 2024 Cisco Systems, Inc. and its affiliates
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# SPDX-License-Identifier: Apache-2.0

"""API response object."""

from __future__ import annotations
from typing import Optional, Generic, Mapping, TypeVar
from pydantic import Field, StrictInt, StrictBytes, BaseModel

T = TypeVar("T")

class ApiResponse(BaseModel, Generic[T]):
    """
    API response object
    """

    status_code: StrictInt = Field(description="HTTP status code")
    headers: Optional[Mapping[str, str]] = Field(None, description="HTTP headers")
    data: T = Field(description="Deserialized data given the data type")
    raw_data: StrictBytes = Field(description="Raw data (HTTP response body)")

    model_config = {
        "arbitrary_types_allowed": True
    }
